use axum::{routing, Router};
use axum_template::{engine::Engine, Key, RenderHtml};
use minijinja::{path_loader, Environment};
use std::net::SocketAddr;

use std::thread;
use std::time::Duration;

mod soundclips;
mod vbplay;

type AppEngine = Engine<Environment<'static>>;

fn play_liv() {
    let file_name = "sounds/example/liv_aaaaaaaa.mp3";
    let tx = vbplay::audio_thread();

    tx.send(vbplay::Command::Play(file_name.into())).unwrap();

    thread::sleep(Duration::from_secs(5));

    tx.send(vbplay::Command::Stop).unwrap();
}

#[tokio::main]
async fn main() {
    let mut jinja = Environment::new();
    jinja.set_loader(path_loader("templates"));

    // Set the address to run our application on
    let addr = SocketAddr::from(([127, 0, 0, 1], 3000));

    // Build our application with a route
    let app = Router::new()
        .route("/", routing::get(home))
        .route("/play/:x", routing::get(play_handler))
        .with_state(Engine::from(jinja));

    println!("listening on {}", addr);

    // Run it with hyper
    axum::Server::bind(&addr)
        .serve(app.into_make_service())
        .await
        .unwrap();
}

async fn home(engine: AppEngine) -> impl axum::response::IntoResponse {
    RenderHtml(Key("index.html".to_owned()), engine, ())
}

async fn root_handler() -> axum::response::Html<&'static str> {
    let html = r#"
        <!DOCTYPE html>
        <html>
        <head>
            <title>Play</title>
            <script>
                function play() {
                    fetch("/play/1");
                }
            </script>
        </head>
        <body>
            <button onclick="play()">Play</button>
        </body>
        </html>
    "#;
    html.into()
}

async fn play_handler(axum::extract::Path(x): axum::extract::Path<u32>) -> String {
    play_liv();
    format!("You sent: {}", x)
}
